################################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################

import os
import sys
import demjson
import datetime

from Products.Five.browser import BrowserView
from Globals import InitializeClass
from App.config import getConfiguration

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
import unix

class StatusView(BrowserView):

    def system_information(self): 
        """ return some system information json-encoded """
        cfg = getConfiguration()
        result = dict()
        result["INSTANCE_HOME"] = cfg.instancehome
        result["port_base"] = cfg.port_base
        result["ip_address"] = cfg.ip_address
        result["debug_mode_cfg"] = cfg.debug_mode
        result["load"] = unix.getLoad()
        result['vmdata_size'] = unix.getVmDataSize()
        result["zserver_threads"] = cfg.zserver_threads
        result['temporary_storage'] = self.context.Control_Panel.Database['temporary']._p_jar.db().getName()
        return demjson.encode(result)

    def get_error_logs(self):
        """ return error_log json-encoded """
        error_log = self.context.error_log
        entries = error_log.getLogEntries()
        for entry in entries:
            entry['time_str'] = datetime.datetime.fromtimestamp(entry['time']).strftime('%d.%m.%y&nbsp;%H:%M:%S')
        entries.sort(lambda x,y: -cmp(x['time'], y['time']))
        return demjson.encode(entries)

InitializeClass(StatusView)



################################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################

from datetime import datetime
import logging
import socket

import Globals
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from BTrees.OOBTree import OOBTree
from App.config import getConfiguration

LOG = logging.getLogger('haufe.monitoring')
ControlPanelId = 'haufe.monitoring control panel'

class Server(object):

    def __init__(self, ip, port, hostname):
        self.ip = ip
        self.port = port
        self.hostname = hostname
        self.url = 'http://%s:%d' % (self.hostname, self.port)
        self.started = datetime.now()
        self.started_str = self.started.strftime('%d.%m.%y %H:%M:%S')

    def __str__(self):
        return ('%s(%s)' % (self.__class__.__name__, self.__dict__))

class ControlPanel(SimpleItem):
    """ haufe.monitoring controlpanel integration """
        
    meta_type = 'HaufeMonitoring ControlPanel'
    id = title = ControlPanelId
    title = 'haufe.monitoring control panel'
    
    security = ClassSecurityInfo()
    security.setDefaultAccess(True)
    
    # Configuration properties
    _properties = ()

    def unregisterZopeInstance(self, host, port):
        try:
            del self.servers[(host, port)]
            LOG.info('Unregistered host %s:%d' % (host, port))
        except KeyError:
            pass

    def registerZopeInstance(self):                 

        if not hasattr(self, 'servers'):
            self.servers = OOBTree()

        port_base = getConfiguration().port_base or 0

        # We can habe multiple servers (although pretty uncommon).
        # Choose the one with the lowest port.
        from ZServer.HTTPServer import zhttp_handler

        candidates =  list()
        servers = getConfiguration().servers
        servers.sort(lambda x,y: cmp(x.port, y.port))
        for server in servers:
            try:            
                if isinstance(server.handlers[0], zhttp_handler):
                    candidates.append(server)
            except:
                pass

        candidate = candidates[0]
        ip_address = candidate.server_port 

        hostname = socket.gethostname()
        ip = socket.gethostbyname(hostname)
        port = port_base + ip_address
        server = Server(ip, port, hostname)
        LOG.info('Registered %s' % server)
        self.servers[(hostname, port)] = server

    def getServers(self):
        servers = list(self.servers.values())
        return servers

Globals.InitializeClass(ControlPanel)



################################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################



import logging
log = logging.getLogger('haufe.monitoring')

from time import time, sleep
from thread import start_new_thread

_runThread = False

def start():
    global _runThread
    if not _runThread:
        _runThread = True
        start_new_thread(_monitor, ())

def stop():
    global _runThead
    _runThread = False

def _monitor():
    while _runThread:
        start = time()
        sleep(1)
        end = time()
        if end - start >= 2:
            log.warning('1s schedule took %ss' % (end-start))
